# Functions for mounting and unmounting block devices
#
# Copyright (C) 2015  Patrick McDermott
#
# This file is part of the ProteanOS Development Kit.
#
# The ProteanOS Development Kit is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# The ProteanOS Development Kit is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the ProteanOS Development Kit.  If not, see
# <http://www.gnu.org/licenses/>.

is_block()
{
	local dev="${1}"
	shift 1

	[ -b "${dev}" ] || return ${?}
	return 0
}

check_block()
{
	local dev="${1}"
	shift 1

	if ! [ -b "${dev}" ]; then
		error "$(get_msg 'block_device_invalid')" "${dev}"
		return 1
	fi

	return 0
}

block_mount()
{
	local dev="${1}"
	shift 1
	local dir=

	if ! check_block "${dev}"; then
		return 2
	fi

	rand
	dir="$(get_vardata_dir 'mount')/block-$(printf '%010d' ${rand_x})"
	if ! mkdir -- "${dir}"; then
		error "$(get_msg 'block_mkdir_fail')" "${dir}"
		return 1
	fi

	if ! mount -- "${dev}" "${dir}"; then
		error "$(get_msg 'block_mount_fail')" "${dev}"
		rmdir -- "${dir}"
		return 1
	fi

	printf '%s' "${dir}"
	return 0
}

block_umount()
{
	local dir="${1}"
	shift 1
	local i=
	local timed_out=

	i=0
	timed_out=false
	while ! umount -- "${dir}"; do
		i=$((${i} + 1))
		if [ ${i} -eq 10 ]; then
			timed_out=true
			break
		fi
		sleep 1
	done

	if ${timed_out}; then
		error "$(get_msg 'block_umount_fail')" "${dir}"
		return 1
	fi

	if ! rmdir -- "${dir}"; then
		error "$(get_msg 'block_rmdir_fail')" "${dir}"
		return 1
	fi

	return 0
}
