# Functions for locking and unlocking mutex objects
#
# Copyright (C) 2014  Patrick McDermott
#
# This file is part of the ProteanOS Development Kit.
#
# The ProteanOS Development Kit is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# The ProteanOS Development Kit is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the ProteanOS Development Kit.  If not, see
# <http://www.gnu.org/licenses/>.

_mutex_trylock()
{
	local mutex="${1}"
	shift 1

	(set -C; printf '%d\n' "${$}" >"${mutex}") 2>/dev/null || return ${?}
	return 0
}

mutex_timedlock()
{
	local mutex="${1}"
	local timeout="${2}"
	shift 2

	while ! _mutex_trylock "${mutex}"; do
		[ ${timeout} -eq 0 ] && return 1
		timeout=$((${timeout} - 1))
		sleep 1
	done

	return 0
}

mutex_unlock()
{
	local mutex="${1}"
	shift 1

	rm -f -- "${mutex}"
}

mutex_is_unlockable()
{
	local mutex="${1}"
	shift 1

	[ "x$(cat -- "${mutex}" 2>/dev/null)" = "x${$}" ] || return ${?}
	return 0
}
